/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.models.entities.objects

import com.alibaba.fastjson.annotation.JSONField
import com.fasterxml.jackson.annotation.JsonIgnoreProperties
import com.persagy.server.datacenter.models.entities.assistant.BIMLocation
import com.persagy.server.datacenter.models.entities.base.BaseInfo
import com.persagy.server.datacenter.models.entities.dict.EquipCategory
import com.persagy.server.datacenter.models.entities.task.SchemeUtils
import com.persagy.service.models.annotations.SCascade
import io.swagger.v3.oas.annotations.media.Schema
import javax.persistence.Column
import javax.persistence.Id
import javax.persistence.Table

/**
 * 设备信息实体类、部件实体类
 *
 * @author  张维新
 */
@Schema(description = "设备信息实体类")
//@Table(name = "equipment")
@JsonIgnoreProperties(ignoreUnknown = true)
open class Equipment : BaseInfo() {

    /** 设备id */
    @Schema(description = "设备id",required = true)
    @Id
    @Column(name = "id")
    var id: String? = null

    /** 设备名称 */
    @Schema(description = "设备名称",required = true)
    @Column(name = "name")
    var name: String? = null

    /** 设备本地编码 */
    @Schema(description = "设备本地编码")
    @Column(name = "local_id")
    var localId: String? = null

    /** 设备本地名称 */
    @Schema(description = "设备本地名称")
    @Column(name = "local_name")
    var localName: String? = null

    /** 二维码图片 */
    @Schema(description = "二维码图片")
    @Column(name = "qr_code_pic")
    var qRCodePic: String? = null

    /** 二维码内容 */
    @Schema(description = "二维码内容")
    @Column(name = "default_qr_code")
    var defaultQRCode: String? = null

    /** 设计图纸中编码 */
    @Schema(description = "设计图纸中编码")
    @Column(name = "cad_id")
    @JSONField(name = "cADID")
    var cadId: String? = null

    /** RFID标识 */
    @Schema(description = "RFID标识")
    @Column(name = "rf_id")
    @JSONField(name = "rFID")
    var rfId: String? = null

    /** BIM模型中编码 */
    @Schema(description = "BIM模型中编码")
    @Column(name = "bim_id")
    var bimId: String? = null

    /** BIM构建编码 */
    @Schema(description = "BIM构建编码")
    @Column(name = "bim_type_id")
    var bimTypeId: String? = null

    /** BIM模型中坐标 */
    @Schema(description = "BIM模型中坐标")
    @Column(name = "bim_location")
    var bimLocation: String? = null

    /** 现场发现人 */
    @Schema(description = "现场发现人")
    @Column(name = "find_people")
    var findPeople: String? = null

    /** BIM模型中坐标 */
    @Schema(description = "BIM模型中坐标json")
    var locationJson: BIMLocation? = null
        get() {
            if (!bimLocation.isNullOrEmpty()){

                val split = bimLocation!!.split(",")
                var bimLocation = BIMLocation()
                if (split.size == 3){
                    bimLocation.x = split[0].toDouble()
                    bimLocation.y = split[1].toDouble()
                    bimLocation.z = split[2].toDouble()
                }else if (split.size == 2){

                    bimLocation.x = split[0].toDouble()
                    bimLocation.y = split[1].toDouble()
                    bimLocation.z = 0.0
                }
                return bimLocation
            }
            return null
        }

    /** 项目id */
    @Schema(description = "项目id",required = true)
    @Column(name = "project_id")
    var projectId: String? = null

    /** 设备类型code */
    @Schema(description = "设备类型code")
    @Column(name = "class_code")
    var classCode: String? = null
    set(value) {
        field = value?.trim()
    }

    /** 设备所在建筑id */
    @Schema(description = "设备所在建筑id")
    @Column(name = "building_id")
    var buildingId: String? = null

    /** 设备所在楼层id */
    @Schema(description = "设备所在楼层id")
    @Column(name = "floor_id")
    var floorId: String? = null

    //    /** 设备关联的资产id */
    @Schema(description = "设备关联的资产id")
    @Column(name = "property_id",insertable = false,updatable = false )
    var propertyId: String? = null

    /** Revit族 */
    @Schema(description = "Revit族 ")
    @Column(name = "family_name")
    var bimFamilyName: String? = null

    /** Revit族类型 */
    @Schema(description = "Revit族类型 ")
    @Column(name = "family_symbol")
    var bimFamilySymbol: String? = null

    /** 设备序号 */
    @Schema(description = "设备序号 ")
    @Column(name = "equip_serial")
    var equipSerial: Double? = null

    /** 信息点 */
    @Schema(description = "信息点")
    @Column(name = "infos")
    var infos: HashMap<String,Any?>? = null

    /** 部件关联设备id */
    @Schema(description = "部件关联设备id")
    @Column(name = "parent_id")
    var parentId: String? = null

    /** 系统id集合 */
    @Schema(description = "系统id集合")
    var systemList: List<String>? = null

    /** 对象类型 */
    @Schema(description = "对象类型")
    @Column(name = "object_type")
    override var objectType: String? = "equipment"

    /** 项目自定义类型 */
    @Schema(description = "项目自定义类型")
    @Column(name = "custom_category")
    var customCategory: String? = null

    /** 任务状态 待验证1、已验证0、未找到-1  */
    @Schema(description = "任务状态-待验证1、已验证0、未找到-1")
    @Column(name = "task_state")
    var taskState: Int? = null

    /** 模型文件id */
    @Schema(description = "模型文件id")
    @Column(name = "model_id")
    var modelId: String? = null

    /** 撕码贴码状态 */
    @Schema(description = "撕码贴码状态")
    @Column(name = "code_type")
    var codeType: Int? = null

    /** 备注 */
    @Schema(description = "备注")
    @Column(name = "note")
    var note: String? = null

    /** 备注 */
    @Schema(description = "客户端id")
    @Column(name = "app_id")
    var appId: String? = null

    /** 方案id */
    @Schema(description = "方案id")
//    @Column(name = "scheme_id")
    var schemeId: String? = null
        get() {
            if (!classCode.isNullOrEmpty()){
                return SchemeUtils.setScheme(classCode!!)
            }
            return null
        }

    /** 资产 */
    @Schema(description = "级联-资产")
    @SCascade(table="property", idColumn="equip_id", childIdColumn="id")
    var property: Property? = null

    /** 建筑   table  对应关系表   idColumn 对应 本类的id    childIdColumn 级联对象的关系id */
    @Schema(description = "级联-建筑")
    @SCascade(table="equipment", idColumn="id", childIdColumn="building_id")
    var building: Building? = null

    /** 级联-楼层   table  对应关系表   idColumn 对应 本类的id    childIdColumn 级联对象的关系id */
    @Schema(description = "级联-楼层")
    @SCascade(table="equipment", idColumn="id", childIdColumn="floor_id")
    var floor: Floor? = null

    /** 级联-项目 */
    @Schema(description = "级联-项目")
    @SCascade(table="equipment", idColumn="id", childIdColumn="project_id")
    var project: Project? = null

    /** 级联-设备分类 */
    @Schema(description = "级联-设备分类")
    @SCascade(table="equipment", idColumn="id", childIdColumn="class_code")
    var equipCategory: EquipCategory? = null

    /** 级联-设备服务的空间的 */
    @Schema(description = "级联-设备服务的空间的")
    @SCascade(table="relationship.r_eq2sp", idColumn="eq_id", childIdColumn="sp_id",filter = "type = 'eq2sp_for'")
    var zoneSpaceForBase: ArrayList<ZoneSpace>? = null

    /** 级联-设备所在的空间的 */
    @Schema(description = "级联-设备所在的空间的")
    @SCascade(table="relationship.r_eq2sp", idColumn="eq_id", childIdColumn="sp_id",filter = "type = 'eq2sp_in'")
    var zoneSpaceInBase: ArrayList<ZoneSpace>? = null

    /** 级联-设备所在的竖井 */
    @Schema(description = "级联-设备所在的竖井")
    @SCascade(table="r_eq_in_sh", idColumn="equip_id", childIdColumn="shaft_id")
    var shaftList: ArrayList<Shaft>? = null

    /** 资产列表 */
    @Schema(description = "资产列表")
    var propertyList: ArrayList<Property>? = null

    /** 级联-部件   table  对应关系表   idColumn 对应 本类的id    childIdColumn 级联对象的关系id */
    @Schema(description = "级联-部件")
    @SCascade(table="equipment", idColumn="parent_id", childIdColumn="id")
    var component: ArrayList<Component>? = null

}