/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.models.entities.objects


import com.persagy.server.datacenter.models.entities.assistant.BIMLocation
import com.persagy.server.datacenter.models.entities.base.BaseInfo
import com.persagy.server.datacenter.models.entities.dict.EquipFamilyCategory
import com.persagy.server.datacenter.models.entities.dict.Family
import com.persagy.server.datacenter.models.entities.task.SchemeUtils
import com.persagy.service.models.annotations.SCascade
import io.swagger.v3.oas.annotations.media.Schema
import javax.persistence.Column
import javax.persistence.Id
import javax.persistence.Table

/**
 * 资产信息实体类
 *
 * @author  张维新
 */
@Schema(description = "资产信息实体类")
@Table(name = "property")
open class Property : BaseInfo(),Comparator<Property> {

    /** 资产id */
    @Schema(description = "资产id", required = true)
    @Id
    @Column(name = "id")
    var id: String? = null

    /** 资产名称 */
    @Schema(description = "资产名称", required = true)
    @Column(name = "name")
    var name: String? = null

    /** 资产本地编码 */
    @Schema(description = "资产本地编码")
    @Column(name = "local_id")
    var localId: String? = null

    /** 资产本地名称 */
    @Schema(description = "资产本地名称")
    @Column(name = "local_name")
    var localName: String? = null

    /** 资产二维码照片 */
    @Schema(description = "资产二维码照片")
    @Column(name = "qr_code")
    var qrCode: String? = null

    /** 设计图纸中编码 */
    @Schema(description = "设计图纸中编码")
    @Column(name = "cad_id")
    var cadId: String? = null

    /** BIM模型中编码 */
    @Schema(description = "BIM模型中编码")
    @Column(name = "bim_id")
    var bimId: String? = null

    /** BIM模型中坐标 */
    @Schema(description = "BIM模型中坐标")
    @Column(name = "bim_location")
    var bimLocation: String? = null


    /** BIM模型中坐标 */
    @Schema(description = "BIM模型中坐标json")
    var locationJson: BIMLocation? = null
        get() {
            if (!bimLocation.isNullOrEmpty()){

                val split = bimLocation!!.split(",")
                var bimLocation = BIMLocation()
                if (split.size == 3){
                    bimLocation.x = split[0].toDouble()
                    bimLocation.y = split[1].toDouble()
                    bimLocation.z = split[2].toDouble()
                }else if (split.size == 2){

                    bimLocation.x = split[0].toDouble()
                    bimLocation.y = split[1].toDouble()
                    bimLocation.z = 0.0
                }
                return bimLocation
            }
            return null
        }

    /** 项目id */
    @Schema(description = "项目id", required = true)
    @Column(name = "project_id")
    var projectId: String? = null

    /** 资产类型code */
    @Schema(description = "资产类型code")
    @Column(name = "family")
    var family: String? = null

    /** 资产所在建筑id */
    @Schema(description = "资产所在建筑id")
    @Column(name = "building_id")
    var buildingId: String? = null

    /** 资产所在楼层id */
    @Schema(description = "资产所在楼层id")
    @Column(name = "floor_id")
    var floorId: String? = null

    /** 资产关联设备id */
    @Schema(description = "关联的设备或部件")
    @Column(name = "equip_id")
    var equipId: String? = null

    /** 设备序号 */
    @Schema(description = "设备序号 ")
    @Column(name = "equip_serial")
    var equipSerial: String? = null

    /** 任务状态 待验证1、已验证0、未找到-1  */
    @Schema(description = "任务状态-待验证1、已验证0、未找到-1")
    @Column(name = "task_state")
    var taskState: Int? = null

    /** 设备部件标志位，1设备，2部件 */
    @Schema(description = "1设备，2部件")
    @Column(name = "type")
    var type: String? = null

    /** 信息点 */
    @Schema(description = "信息点")
    @Column(name = "infos")
    var infos: HashMap<String,Any?>? = null

    /** 对象类型 */
    @Schema(description = "对象类型")
    @Column(name = "object_type")
    override var objectType: String? = "property"

    /** 位置信息 */
    @Schema(description = "位置信息")
    @Column(name = "position")
    var position: String? = null

    /** 贴码状态 */
    @Schema(description = "贴码状态")
    @Column(name = "code_type")
    var codeType: Long? = null

    /** 选择状态 */
    @Schema(description = "选择状态")
    @Column(name = "checked")
    var checked: Boolean? = null

    /** 空间id */
    @Schema(description = "空间id")
    @Column(name = "space_id")
    var spaceId: String? = null

    /** 备注 */
    @Schema(description = "备注")
    @Column(name = "note")
    var note: Boolean? = null

    /** 点位id */
    @Schema(description = "点位id")
    @Column(name = "point_id")
    var pointId: String? = null

    /** 区分是哪里创建的资产 app  ,web */
    @Schema(description = "区分是哪里创建的资产 app  ,web")
    @Column(name = "source")
    var source: String? = null

    /** 模型文件id */
    @Schema(description = "模型文件id")
    @Column(name = "model_id")
    var modelId: String? = null

    @Schema(description = "方案id")
//    @Column(name = "scheme_id")
    var schemeId: String? = null
        get() {
            if (!family.isNullOrEmpty()){
                return SchemeUtils.setScheme(family!!)
            }
            return null
        }

    /** 设备 */
    @Schema(description = "级联-设备")
    @SCascade(table="property", idColumn="id", childIdColumn="equip_id")
    var equipment: Equipment? = null

    /** 建筑   table  对应关系表   idColumn 对应 本类的id    childIdColumn 级联对象的关系id */
    @Schema(description = "级联-建筑")
    @SCascade(table="property", idColumn="id", childIdColumn="building_id")
    var building: Building? = null

    /** 级联-楼层   table  对应关系表   idColumn 对应 本类的id    childIdColumn 级联对象的关系id */
    @Schema(description = "级联-楼层")
    @SCascade(table="property", idColumn="id", childIdColumn="floor_id")
    var floor: Floor? = null

    /** 级联-项目 */
    @Schema(description = "级联-项目")
    @SCascade(table="property", idColumn="id", childIdColumn="project_id")
    var project: Project? = null

    /** 级联-资产设备族 */
    @Schema(description = "级联-资产设备族")
    @SCascade(table="property", idColumn="id", childIdColumn="family")
    var equipFamilyList: ArrayList<EquipFamilyCategory>? = null

    /** 级联-资产设备族名称 */
    @Schema(description = "级联-资产设备族名称")
    @SCascade(table="property", idColumn="id", childIdColumn="family")
    var familyName: Family? = null

    /** 级联-资产所在的空间的 */
    @Schema(description = "级联-设备所在的空间的")
    @SCascade(table="r_pe_in_sp_base", idColumn="equip_id", childIdColumn="space_id")
    var zoneSpaceInBase: ArrayList<ZoneSpace>? = null

    /** 距离*/
    @Schema(description = "距离")
    var distance: Double? = null

    override fun compare(o1: Property?, o2: Property?): Int {
        return if (o1!!.distance!=null&&o2!!.distance!=null&&o1.distance!!>o2.distance!!){
            1
        }else{
            -1
        }
    }

} // Class Property