/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.models.entities.objects

import com.alibaba.fastjson.annotation.JSONField
import com.persagy.server.datacenter.models.entities.assistant.PointPosition
import com.persagy.server.datacenter.models.entities.base.BaseInfo
import com.persagy.service.models.annotations.SCascade
import io.swagger.v3.oas.annotations.media.Schema
import javax.persistence.Column
import javax.persistence.Id
import javax.persistence.Table

/**
 * 竖井信息实体类
 *
 * @author  张维新
 */
@Schema(description = "竖井信息实体类")
@Table(name = "shaft")
class Shaft : BaseInfo() {

    /** 竖井id */
   @Schema(description = "竖井id")
    @Id
    @Column(name = "id")
    var id: String? = null

    /** 竖井名称 */
   @Schema(description = "竖井名称")
    @Column(name = "name")
    var name: String? = null

    /** 竖井本地编码 */
   @Schema(description = "竖井本地编码")
    @Column(name = "local_id")
    var localId: String? = null

    /** 竖井本地名称 */
   @Schema(description = "竖井本地名称")
    @Column(name = "local_name")
    var localName: String? = null

    /** BIM模型中编码 */
   @Schema(description = "BIM模型中编码")
    @Column(name = "bim_id")
    var bimId: String? = null

    /** 设计图纸中编码 */
    @Schema(description = "设计图纸中编码")
    @Column(name = "cad_id")
    @JSONField(name = "cADID")
    var cadId: String? = null

    /** 二维码图片 */
   @Schema(description = "二维码图片")
    @Column(name = "qr_code_pic")
    var qRCodePic: String? = null

    /** 二维码内容 */
   @Schema(description = "二维码内容")
    @Column(name = "default_qr_code")
    var defaultQRCode: String? = null

    /** 项目id */
   @Schema(description = "项目id")
    @Column(name = "project_id")
    var projectId: String? = null

    /** BIM模型中坐标 */
   @Schema(description = "BIM模型中坐标")
    @Column(name = "bim_location")
    var bimLocation: String? = null

    /** 竖井顺序码 */
   @Schema(description = "竖井顺序码")
    @Column(name = "shaft_sequenceId")
    var shaftSequenceId: Int? = null

    /** 信息点 */
   @Schema(description = "信息点")
    @Column(name = "infos")
    var infos: HashMap<String, Any?>? = null

    /** BIM模型中轮廓坐标 */
   @Schema(description = "BIM模型中轮廓坐标")
    @Column(name = "outline")
    var outline: ArrayList<PointPosition>? = null

    /** 对象类型 */
   @Schema(description = "对象类型")
    @Column(name = "object_type")
    override var objectType: String? = "shaft"

    /** 级联-竖井内的设备 */
   @Schema(description = "级联-竖井内的设备")
    @SCascade(table="r_eq_in_sh", idColumn="shaft_id", childIdColumn="equip_id")
    var equipmentList: ArrayList<Equipment>? = null

    /** 级联-竖井内的系统 */
    @Schema(description = "级联-竖井内的系统")
    @SCascade(table="r_sh_in_bd", idColumn="shaft_id", childIdColumn="building_id")
    var buildingList: ArrayList<Building>? = null


    /** 级联-竖井内的系统 */
   @Schema(description = "级联-竖井内的系统")
    @SCascade(table="r_sy_in_sh", idColumn="shaft_id", childIdColumn="sys_id")
    var systemList: ArrayList<GeneralSystem>? = null

    /** 贯通关系   self = true  同样的对象查询时 id可以互相调换查询   */
   @Schema(description = "级联-贯通关系")
    @SCascade(table="r_sh_through_sh", idColumn="shaft_id", childIdColumn="shaft_other_id", self = true)
    var shaftThroughList: ArrayList<Shaft>? = null

    /** 竖井内的业务空间  self = true  同样的对象查询时 id可以互相调换查询   */
   @Schema(description = "级联-竖井内的业务空间")
    @SCascade(table="relationship.r_sh2sp", idColumn="id1", childIdColumn="id2", self = false,filter = "type = 'sh2sp'")
    var spaceList: ArrayList<ZoneSpace>? = null


} // Class Shaft