/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.models.entities.objects

import com.alibaba.fastjson.annotation.JSONField
import com.persagy.server.datacenter.models.entities.base.BaseInfo
import com.persagy.service.models.annotations.SCascade
import io.swagger.v3.oas.annotations.media.Schema
import javax.persistence.Column
import javax.persistence.Id
import javax.persistence.Table

/**
 * 租户
 *
 * @author 李井强
 */
@Schema(description = "租户信息实体类")
@Table(name = "tenant")
class Tenant : BaseInfo() {

    /** 租户id */
    @Schema(description = "租户id")
    @Id
    @Column(name = "id")
    var id: String? = null

    /** 租户名称 */
    @Schema(description = "租户名称")
    @Column(name = "name")
    var name: String? = null

    /** 租户本地编码 */
    @Schema(description = "租户本地编码")
    @Column(name = "local_id")
    var localId: String? = null

    /** 租户本地名称 */
    @Schema(description = "租户本地名称")
    @Column(name = "local_name")
    var localName: String? = null

    /** 租户本地编码2 */
    @Schema(description = "租户本地编码2")
    @Column(name = "local_id2")
    var localId2: String? = null

    /** 租户本地名称2 */
    @Schema(description = "租户本地名称2")
    @Column(name = "local_name2")
    var localName2: String? = null

    /** 项目id */
    @Schema(description = "项目id")
    @Column(name = "project_id")
    var projectId: String? = null

    /** 信息点 */
    @Schema(description = "信息点")
    @Column(name = "infos")
    var infos: HashMap<String,Any?>? = null


    /** 对象类型 */
    @Schema(description = "对象类型")
    @Column(name = "object_type")
    override var objectType: String? = "tenant"


    /** 租户租赁的租户业务空间  self = true  同样的对象查询时 id可以互相调换查询   */
    @Schema(description = "级联-租户租赁的租户业务空间")
    @SCascade(table="relationship.r_tn2sp", idColumn="tn_id", childIdColumn="sp_id", self = false,filter = "type = 'tn2sp' and zone_type = 'TenantZone'")
    var zoneTenantList: ArrayList<ZoneSpace>? = null

    /** 租户租赁的默认业务空间  self = true  同样的对象查询时 id可以互相调换查询   */
    @Schema(description = "级联-租户租赁的默认业务空间")
    @SCascade(table="relationship.r_tn2sp", idColumn="tn_id", childIdColumn="sp_id", self = false,filter = "type = 'tn2sp' and zone_type = 'GeneralZone'")
    var zoneGeneraltList: ArrayList<ZoneSpace>? = null

} // Class Tenant