/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.models.entities.objects

import com.fasterxml.jackson.annotation.JsonIgnoreProperties
import com.persagy.server.datacenter.models.entities.assistant.PointPosition
import com.persagy.server.datacenter.models.entities.base.BaseInfo
import io.swagger.v3.oas.annotations.media.Schema
import javax.persistence.Column
import javax.persistence.Id
import javax.persistence.Table

/**
 * 元空间信息实体类
 *
 * @author  张维新
 */
@Schema(description = "元空间信息实体类")
@JsonIgnoreProperties(ignoreUnknown = true)
@Table(name = "zone_ispace")
open class ZoneIspace : BaseInfo() {

    /** 空间ID */
    @Schema(description = "空间ID")
    @Id
    @Column(name = "id")
    var id: String? = null

    /** 空间名称 */
    @Schema(description = "空间名称")
    @Column(name = "name")
    var name: String? = null

    /** 空间本地编码 */
    @Schema(description = "空间本地编码")
    @Column(name = "local_id")
    var localId: String? = null

    /** 空间本地名称 */
    @Schema(description = "空间本地名称")
    @Column(name = "local_name")
    var localName: String? = null

    /** 项目id */
    @Schema(description = "项目id")
    @Column(name = "project_id")
    var projectId: String? = null

    /** 建筑id */
    @Schema(description = "建筑id")
    @Column(name = "building_id")
    var buildingId: String? = null

    /** 楼层id */
    @Schema(description = "楼层id")
    @Column(name = "floor_id")
    var floorId: String? = null

    /** 空间二维码图片 */
    @Schema(description = "空间二维码图片")
    @Column(name = "qr_code")
    var qrCode: String? = null

    /** BIM模型中编码 */
    @Schema(description = "BIM模型中编码")
    @Column(name = "bim_id")
    var bimId: String? = null

    /** BIM模型中坐标 */
    @Schema(description = "BIM模型中坐标")
    @Column(name = "bim_location")
    var bimLocation: String? = null

    /** 空间功能区类型 */
    @Schema(description = "空间功能区类型")
    @Column(name = "func_type")
    var funcType: String? = null

    /** 进深 */
    @Schema(description = "进深")
    @Column(name = "depth")
    var depth: Double? = null

    /** 面宽 */
    @Schema(description = "面宽")
    @Column(name = "width")
    var width: Double? = null

    /** 建筑面积 */
    @Schema(description = "建筑面积")
    @Column(name = "build_area")
    var buildArea: Double? = null

    /** 使用面积 */
    @Schema(description = "使用面积")
    @Column(name = "net_area")
    var netArea: Double? = null

    /** 高度 */
    @Schema(description = "高度")
    @Column(name = "height")
    var height: Double? = null

    /** 配电容量 */
    @Schema(description = "配电容量")
    @Column(name = "elec_cap")
    var elecCap: Double? = null

    /** 污染等级 */
    @Schema(description = "污染等级")
    @Column(name = "pollution_degree")
    var pollutionDegree: String? = null

    /** 备注文字 */
    @Schema(description = "备注文字")
    @Column(name = "intro")
    var intro: String? = null

    /** 信息点 */
    @Schema(description = "信息点")
    @Column(name = "infos")
    var infos: HashMap<String, Any?>? = null

    @Schema(description = "对象类型")
    @Column(name = "object_type")
    override var objectType: String? = "ispace"

    /** 模型文件id */
    @Schema(description = "模型文件id")
    @Column(name = "model_id")
    var modelId: String? = null

   @Schema(description = "轮廓线")
   @Column(name = "outline")
   var outline: ArrayList<ArrayList<PointPosition>>? = null

   /* @Schema(description = "轮廓线")
    val outLine: List<LinkedHashMap<String, Any>>? = null*/

}