package com.persagy.labsl.controller

import com.persagy.labsl.models.entities.tpt.Node
import com.persagy.labsl.services.NodeService
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.requests.SUpdateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

/**
 * 图例节点
 *
 * @author wx  <zhangweixin@sagacloud.com>
 */
@Tag(name = "002、系统图设备节点，与数据中台设备实例对应关系（绑定信息点在该设备上进行）- 系统图下子对象")
@RestController
@RequestMapping("/node")
class NodeController {
    companion object {
        /** 日志 */
        private val logger = LoggerFactory.getLogger(NodeController::class.java)


    } // Companion object

    /**
     * 图例节点
     *
     * @param request   待创建图对象列表
     * @return  创建完成图例节点对象列表
     */
   @Operation(summary="创建设备节点", description ="id 为系统图的图对象id，graphId 为系统图的图id ,nodeId 为当前节点的id，id 和 graphId 必须填为某一个系统图中的 id，nodeId 是当前要创建的对象id不传会自动生成（32位uuid）")
    @PostMapping("/create")
    fun create(@RequestBody request: SCreateRequest<Node>): SCreateResponse<Node> {
        return NodeService.creates(request)
    } // Function create()

    /**
     * 根据id删除图例节点
     *
     * @param idList  id数组
     * @return 删除的结果信息
     */
   @Operation(summary = "根据id删除图例节点信息", description = "创建中有介绍id的说明请查看，删除时必须传入 id、graphId、nodeId 才能确认删除的是当前的设备节点")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody idList: ArrayList<Node>): SBaseResponse {
        return NodeService.deletes(idList)
    } // Function delete()

    /**
     * 更新图例节点信息
     *
     * @param  request   更新的内容对象
     * @return 更新的结果
     */
   @Operation(summary = "更新图例节点信息", description = "创建中有介绍 id 的说明请查看，id、graphId、nodeId 为必填信息点，参考创建后的对象信息点，然后修改了哪个信息点传哪个信息点，详细的使用方式参考 web 端接口使用规范")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<Node>): SBaseResponse {
        return NodeService.updateList(request)
    } // Function update()

    /**
     * 查询图例节点信息
     *
     * @param  request 查询条件
     * @return 查询结果
     */
   @Operation(summary = "查询图例节点信息", description = "创建中有介绍 id 的说明请查看,注意 id、graphId、nodeId 的区别，示例 查询某一个对象：filters = id ='值';graphId='值';nodeId='值' （其他的组合方式详见后端开发文档 web 端接口使用规范）")
    @PostMapping(value = ["/query"])
    fun query(@RequestBody request: SQueryRequest): SQueryResponse<Node> {
        return NodeService.pageQuery(request)
    } // Function query()

} // Class NodeController