package com.persagy.labsl.controller

import com.persagy.labsl.models.entities.tpt.Relation
import com.persagy.labsl.services.RelationService
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.requests.SUpdateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

/**
 * 管线对象
 *
 * @author wx  <zhangweixin@sagacloud.com>
 */
@Tag(name = "004、系统图管线信息（对应中台中的关系）- 系统图下子对象")
@RestController
@RequestMapping("/relation")
class RelationController {

    /**
     * 管线创建
     *
     * @param request    待创建管线对象列表
     * @return  创建完成图例节点对象列表
     */
   @Operation(summary="管线创建", description ="id 为系统图的图对象id，graphId 为系统图的图id ,relationId 为当前管线对象的id，id 和 graphId 必须填为某一个系统图中的id，relationId 是当前要创建的对象 id 不传会自动生成（32位uuid）")
    @PostMapping("/create")
    fun create(@RequestBody request: SCreateRequest<Relation>): SCreateResponse<Relation> {
        return RelationService.creates(request)
    }

    /**
     * 根据id删除管线
     *
     * @param  idList    id数组
     * @return 删除的结果信息
     */
   @Operation(summary = "根据id删除管线信息", description = "创建中有介绍id的说明请查看，删除时必须传入 id、graphId、relationId 才能确认删除的是当前的标识对象")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody idList: ArrayList<Relation>): SBaseResponse {
        return RelationService.deletes(idList)
    }

    /**
     * 更新管线信息
     *
     * @param  request   更新的内容对象
     * @return 更新的结果
     */
   @Operation(summary = "更新管线信息", description = "创建中有介绍id的说明请查看，id、graphId、relationId 为必填信息点，参考创建后的对象信息点,修改哪个信息点直接写在对象里面即可，详细的使用方式参考 web 端接口使用规范")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<Relation>): SBaseResponse {
        return RelationService.updateList(request)
    }

    /**
     * 查询管线信息
     *
     * @param  request     查询条件
     * @return  查询结果
     */
   @Operation(summary = "查询管线信息", description = "创建中有介绍id的说明请查看,注意 id、graphId、relationId 的区别，示例 查询某一个对象：filters = id ='值';graphId='值';relationId='值' （其他的组合方式详见后端开发文档 web 端接口使用规范）")
    @PostMapping(value = ["/query"])
    fun query(@RequestBody request: SQueryRequest): SQueryResponse<Relation> {
        return RelationService.pageQuery(request)
    }

}