package cn.sagacloud.server.algorithm.controllers.modelEntity

import cn.sagacloud.server.algorithm.models.entities.InfoPointComp
import com.sybotan.mybatis.SMybatisDao
import com.sybotan.service.SObjectService
import com.sybotan.service.models.requests.SCountRequest
import com.sybotan.service.models.requests.SCreateRequest
import com.sybotan.service.models.requests.SQueryRequest
import com.sybotan.service.models.requests.SUpdateRequest
import com.sybotan.service.models.responses.SBaseResponse
import com.sybotan.service.models.responses.SCountResponse
import com.sybotan.service.models.responses.SCreateResponse
import com.sybotan.service.models.responses.SQueryResponse
import io.swagger.annotations.Api
import io.swagger.annotations.ApiOperation
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

/**
 *
 * @author  Cainga
 * @date  2021/1/26 16:39
 */
@Api(tags = ["信息点映射接口"])
@RestController
@RequestMapping("/infoPointComp")
class InfoPointCompController {

    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(InfoPointCompController::class.java)
    } // Companion object
    
    val infoPointCompService = SObjectService(SMybatisDao(InfoPointComp::class.java))  // 信息点映射

    /**
     * 创建信息点映射
     *
     * @param   entity     信息点映射实体
     * @return  创建结果应答信息
     */
    @ApiOperation(value = "创建信息点映射", notes = "")
    @PostMapping(value = ["/create"])
    fun create(@RequestBody entity: SCreateRequest<InfoPointComp>): SCreateResponse<InfoPointComp> {
        return infoPointCompService.createList(entity)
    } // Function create()

    /**
     * 删除信息点映射
     *
     * @param   idList      信息点映射ID列表
     * @return  删除结果应答信息
     */
    @ApiOperation(value = "删除信息点映射", notes = "")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody idList: ArrayList<InfoPointComp>): SBaseResponse {
        return infoPointCompService.deleteByKeysList(idList)
    } // Function delete()

    /**
     * 更新信息点映射
     *
     * @param   request     更新请求
     * @return  更新结果应答信息
     */
    @ApiOperation(value = "更新信息点映射", notes = "")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<InfoPointComp>): SBaseResponse {
        return infoPointCompService.updateList(request)
    } // Function update()

    /**
     * 查询信息点映射
     *
     * @param   request     查询请求
     * @return  查询结果应答信息
     */
    @ApiOperation(value = "查询信息点映射", notes = "")
    @PostMapping(value = ["/query"])
    fun query(@RequestBody request: SQueryRequest): SQueryResponse<InfoPointComp> {
        return infoPointCompService.pageQuery(request)
    } // Function query()

    /**
     * 根据条件查询统计数量
     */
    @ApiOperation(value = "根据条件查询统计数量", notes = "")
    @PostMapping(value = ["/count"])
    fun count(@RequestBody request: SCountRequest): SCountResponse {
        return infoPointCompService.count(request)
    } // Function count()
}