package cn.sagacloud.server.algorithm.controllers.modelEntity

import cn.sagacloud.server.algorithm.models.entities.Other
import cn.sagacloud.server.algorithm.services.BaseDataService
import com.sybotan.service.models.requests.SCountRequest
import com.sybotan.service.models.requests.SCreateRequest
import com.sybotan.service.models.requests.SQueryRequest
import com.sybotan.service.models.requests.SUpdateRequest
import com.sybotan.service.models.responses.SBaseResponse
import com.sybotan.service.models.responses.SCountResponse
import com.sybotan.service.models.responses.SCreateResponse
import com.sybotan.service.models.responses.SQueryResponse
import io.swagger.annotations.Api
import io.swagger.annotations.ApiOperation
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

/**
 *
 *
 * @author jxing
 */
@Api(tags = ["构件-其他对象接口"])
@RestController
@RequestMapping("/other")
class OtherController {

    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(ConnectorController::class.java)
    } // Companion object

    val dataService = BaseDataService

    /**
     * 创建连接件
     *
     * @param   entity     实体
     * @return  创建结果应答信息
     */
    @ApiOperation(value = "创建连接件", notes = "")
    @PostMapping(value = ["/create"])
    fun create(@RequestBody entity: SCreateRequest<Other>): SCreateResponse<Other> {
        return dataService.otherService.createList(entity)
    } // Function create()

    /**
     * 删除其他对象
     *
     * @param   idList      其他对象ID列表
     * @return  删除结果应答信息
     */
    @ApiOperation(value = "删除其他对象", notes = "")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody idList: ArrayList<Other>): SBaseResponse {
        return dataService.otherService.deleteByKeysList(idList)
    } // Function delete()

    /**
     * 更新其他对象
     *
     * @param   request     更新请求
     * @return  更新结果应答信息
     */
    @ApiOperation(value = "更新其他对象", notes = "")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<Other>): SBaseResponse {
        return dataService.otherService.updateList(request)
    } // Function update()

    /**
     * 查询其他对象
     *
     * @param   request     查询请求
     * @return  查询结果应答信息
     */
    @ApiOperation(value = "查询其他对象", notes = "")
    @PostMapping(value = ["/query"])
    fun query(@RequestBody request: SQueryRequest): SQueryResponse<Other> {
        return dataService.otherService.pageQuery(request)
    } // Function query()

    /**
     * 根据条件查询统计数量
     */
    @ApiOperation(value = "根据条件查询统计数量", notes = "")
    @PostMapping(value = ["/count"])
    fun count(@RequestBody request: SCountRequest): SCountResponse {
        return dataService.otherService.count(request)
    } // Function count()
}