package cn.sagacloud.server.algorithm.controllers.modelEntity

import cn.sagacloud.server.algorithm.models.entities.Project
import com.sybotan.mybatis.SMybatisDao
import com.sybotan.service.SObjectService
import com.sybotan.service.models.requests.SCountRequest
import com.sybotan.service.models.requests.SCreateRequest
import com.sybotan.service.models.requests.SQueryRequest
import com.sybotan.service.models.requests.SUpdateRequest
import com.sybotan.service.models.responses.SBaseResponse
import com.sybotan.service.models.responses.SCountResponse
import com.sybotan.service.models.responses.SCreateResponse
import com.sybotan.service.models.responses.SQueryResponse
import io.swagger.annotations.Api
import io.swagger.annotations.ApiOperation
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

/**
 *
 * @author  Cainga
 * @date  2021/1/26 16:39
 */
@Api(tags = ["项目配置接口"])
@RestController
@RequestMapping("/project")
class ProjectController {

    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(ProjectController::class.java)
    } // Companion object
    
    val projectService = SObjectService(SMybatisDao(Project::class.java))  // 项目

    /**
     * 创建项目
     *
     * @param   entity     项目实体
     * @return  创建结果应答信息
     */
    @ApiOperation(value = "创建项目", notes = "")
    @PostMapping(value = ["/create"])
    fun create(@RequestBody entity: SCreateRequest<Project>): SCreateResponse<Project> {
        return projectService.createList(entity)
    } // Function create()

    /**
     * 删除项目
     *
     * @param   idList      项目ID列表
     * @return  删除结果应答信息
     */
    @ApiOperation(value = "删除项目", notes = "")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody idList: ArrayList<Project>): SBaseResponse {
        return projectService.deleteByKeysList(idList)
    } // Function delete()

    /**
     * 更新项目
     *
     * @param   request     更新请求
     * @return  更新结果应答信息
     */
    @ApiOperation(value = "更新项目", notes = "")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<Project>): SBaseResponse {
        return projectService.updateList(request)
    } // Function update()

    /**
     * 查询项目
     *
     * @param   request     查询请求
     * @return  查询结果应答信息
     */
    @ApiOperation(value = "查询项目", notes = "")
    @PostMapping(value = ["/query"])
    fun query(@RequestBody request: SQueryRequest): SQueryResponse<Project> {
        return projectService.pageQuery(request)
    } // Function query()

    /**
     * 根据条件查询统计数量
     */
    @ApiOperation(value = "根据条件查询统计数量", notes = "")
    @PostMapping(value = ["/count"])
    fun count(@RequestBody request: SCountRequest): SCountResponse {
        return projectService.count(request)
    } // Function count()
}