/*
 * ********************************************************************************************************************
 *
 *                     :*$@@%$*:                         ;:                ;;    ;;
 *                   :@@%!  :!@@%:                       %!             ;%%@@%$ =@@@@@@@%;     @%@@@%%%%@@@@@
 *                  :@%;       :$=                       %%$$$%$$         ;$$  ;$@=   !@$
 *                  =@!                                  %!              @ $=;%   !@@@%:      !$$$$$$$$$$$$$$=
 *                  =@*                                  %!              @ $= % %@=   =%@!      %=
 *             *$%%! @@=        ;=$%%%$*:                %!              @ $= % =%%%%%%@$      *%:         =%
 *           %@@!:    !@@@%=$@@@@%!  :*@@$:              %!              @ $= % $*     ;@      @*          :%*
 *         ;@@!          ;!!!;:         ;@%:      =======@%========*     @ $$ % $%*****$@     :@$=*********=@$
 *         $@*   ;@@@%=!:                *@*
 *         =@$    ;;;!=%@@@@=!           =@!
 *          %@$:      =@%: :*@@@*       %@=                    Copyright (c) 2016-2019.  北京上格云技术有限公司
 *           ;%@@$=$@@%*       *@@@$=%@@%;
 *              ::;::             ::;::                                              All rights reserved.
 *
 * ********************************************************************************************************************
 */

package cn.sagacloud.server.algorithm.services.json

import cn.sagacloud.server.algorithm.models.entities.*
import cn.sagacloud.server.algorithm.models.jsonAnalyzer.AllComponents
import cn.sagacloud.server.algorithm.models.jsonAnalyzer.BaseObj
import cn.sagacloud.server.algorithm.models.modelFile.ModelFile
import cn.sagacloud.server.algorithm.services.BaseDataService
import cn.sagacloud.server.algorithm.services.CommonService
import cn.sagacloud.server.algorithm.services.endpoint.UploadService
import com.alibaba.fastjson.JSONObject
import com.alibaba.fastjson.util.ParameterizedTypeImpl
import com.fasterxml.jackson.core.JsonParser
import com.fasterxml.jackson.databind.DeserializationFeature
import com.fasterxml.jackson.databind.ObjectMapper
import com.persagy.revit.services.json.EquipmentAnalyzer
import com.sybotan.database.SFilter
import com.sybotan.service.SObjectService
import com.sybotan.service.models.enums.SResponseType
import com.sybotan.service.models.responses.SBaseResponse
import org.apache.commons.lang3.ObjectUtils
import org.apache.commons.lang3.StringUtils
import org.apache.commons.text.StringEscapeUtils
import org.springframework.stereotype.Component
import java.io.InputStream
import java.lang.reflect.Type


/**
 * 解析revit-json服务
 * 模型识别，空间或者房间读取revit.project  表得配置
 *
 * @author jxing
 */
@Suppress("UNCHECKED_CAST")
@Component
open class JSONAnalyzerService {

//    private var doorService = SObjectService(SMybatisDao(Door::class.java))
//    private var windowService = SObjectService(SMybatisDao(Window::class.java))
//    private var columnService = SObjectService(SMybatisDao(Column::class.java))
//    private var wallService = SObjectService(SMybatisDao(Wall::class.java))
//    private var virtualWallService = SObjectService(SMybatisDao(VirtualWall::class.java))
//    private var spaceService = SObjectService(SMybatisDao(Space::class.java))
//    private var modelService = SObjectService(SMybatisDao(ModelFile::class.java))
//    private var boundarySegmentService = SObjectService(SMybatisDao(BoundarySegment::class.java))

    var dataService = BaseDataService

    /**
     * 组合泛型具体类型组合成大类型
     */
    fun buildType(vararg types: Type): Type? {
        var beforeType: ParameterizedTypeImpl? = null
        if (types.isNotEmpty()) {
            for (i in types.size - 1 downTo 1) {
                beforeType = ParameterizedTypeImpl(arrayOf<Type>(if (beforeType == null) types[i] else beforeType), null, types[i - 1])
            }
        }
        return beforeType
    }


    /**
     * 获取jsonz文件内的模型id
     */
    fun getModelIdInJSON(json: JSONObject): String? {
        val mapper = ObjectMapper()
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false)
        val baseObj = mapper.readValue<BaseObj<AllComponents>>(json.toString(), mapper.typeFactory.constructParametricType(BaseObj::class.java, AllComponents::class.java))
        // 验证是否baseObj.modelId合法, 不合法不允许入库
        val modelFile = checkModelId(baseObj.modelId) ?: return null
        return modelFile.id
    }

    fun handleSingleJSON(json: JSONObject): BaseObj<AllComponents>? {
        val mapper = ObjectMapper()
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false)
        // mapper.configure(JsonParser.Feature.ALLOW_SINGLE_QUOTES,true)
        //val baseObj = mapper.readValue<BaseObj<AllComponents>>(json.toString(), mapper.typeFactory.constructParametricType(BaseObj::class.java, AllComponents::class.java))
        val baseObj = mapper.readValue<BaseObj<AllComponents>>(mapper.writeValueAsString(json),
                mapper.typeFactory.constructParametricType(BaseObj::class.java, AllComponents::class.java))
        return baseObj
    }


    /**
     * 处理上传的json
     */
    fun handleSingleJSON(baseObj: BaseObj<AllComponents>, bimIdPrefix: String): String? {
//        val baseObj = json.toJavaObject<BaseObj<AllComponents>>(buildType(BaseObj::class.java, AllComponents::class.java)!!)
//        val mapper = ObjectMapper()
//        val baseObj = mapper.readValue<BaseObj<AllComponents>>(json.toString(), mapper.typeFactory.constructParametricType(BaseObj::class.java, AllComponents::class.java))
        // 验证是否baseObj.modelId合法, 不合法不允许入库
        val modelFile = checkModelId(baseObj.modelId) ?: return "ModelId ${baseObj.modelId} 不存在!"

        val checkProject = checkProject(modelFile.projectId)
        val projectConfig = getProject(modelFile.projectId)

        var result = true
        // 获取model, 更新字段
        result = result and ModelAnalyzer.analyze(dataService.modelFileService, modelFile, baseObj)
        if (!result) {
            return "获取model异常"
        }
        // 处理VirtualWall, 填充数据, replace插入表
        result = result and BaseBehavier.analyze(
                dataService.virtualWallService as SObjectService<BaseComponent>,
                baseObj,
                baseObj.elements!!.virtualWalls as ArrayList<BaseComponent>?
        )
        if (!result) {
            return "处理VirtualWall异常"
        }
        // 处理Wall, 填充数据, replace插入表
        result = result and BaseBehavier.analyze(
                dataService.wallService as SObjectService<BaseComponent>,
                baseObj,
                baseObj.elements!!.walls as ArrayList<BaseComponent>?
        )
        if (!result) {
            return "处理Wall异常"
        }
        // 处理door, 填充数据, 填充wallId, replace插入表
        result = result and DoorWindowAnalyzer.analyze(
                dataService.doorService as SObjectService<WindowDoorBase>,
                baseObj,
                baseObj.elements!!.doors as ArrayList<WindowDoorBase>?
        )   // 依赖 wall
        if (!result) {
            return "处理door异常"
        }
        // 处理Window, 填充数据, 填充wallId, replace插入表
        result = result and DoorWindowAnalyzer.analyze(
                dataService.windowService as SObjectService<WindowDoorBase>,
                baseObj,
                baseObj.elements!!.windows as ArrayList<WindowDoorBase>?
        )  // 依赖 wall
        if (!result) {
            return "处理Window异常"
        }
        // 处理column, 填充数据, replace插入表
        result = result and BaseBehavier.analyze(
                dataService.columnService as SObjectService<BaseComponent>,
                baseObj,
                baseObj.elements!!.columns as ArrayList<BaseComponent>?
        )
        if (!result) {
            return "处理column异常"
        }
        // 处理Space, 填充数据, replace插入表


        //---------------------------------------------------------
        //12.20临时演示方案，room当作space
        /** 模型识别，空间或者房间读取revit.project  表得配置 */
        if(projectConfig!=null&&projectConfig?.roomSpace == "room"){
            result = result and Space2Analyzer.analyze(dataService.spaceService, baseObj, baseObj.elements!!.rooms)
            if (!result) {
                return "处理Room异常"
            }
        }else{
            result = result and SpaceAnalyzer.analyze(dataService.spaceService, baseObj, baseObj.elements!!.spaces)
            if (!result) {
                return "处理Space异常"
            }
        }


//        //处理Room, 填充数据, replace插入表
//        result = result and RoomAnalyzer.analyze(dataService.roomService, baseObj, baseObj.elements!!.rooms)
//        if (!result) {
//            return "处理Room异常"
//        }


        // 处理BoundarySegment, 填充数据, replace方式插入表
        result = result and BoundarySegmentAnalyzer.analyze(
                dataService.boundarySegmentService,
                baseObj,
                baseObj.elements!!.boundarySegments
        )
        if (!result) {
            return "处理BoundarySegment异常"
        }

        var pipes = baseObj.elements!!.pipes as ArrayList<BaseComponent>?
        var flexPipes = baseObj.elements!!.flexPipes as ArrayList<BaseComponent>?
        var ducts = baseObj.elements!!.ducts as ArrayList<BaseComponent>?
        var flexDucts = baseObj.elements!!.flexDucts as ArrayList<BaseComponent>?
        var connectors = baseObj.elements!!.connectors


        if(checkProject){
            //修改万达管道名称对应标准管道名称
            val pdCompService = dataService.pdCompService // 管道对照
            val pdComps = pdCompService.selectAll().exec()
            val map = HashMap<String, String>()
            for (pdComp in pdComps) {
                map[pdComp.pPdName!!] = pdComp.pdName!!
            }
            pipes = pipeConvert(null, baseObj.elements!!.pipes,map)
            flexPipes = pipeConvert(null, baseObj.elements!!.flexPipes,map)
            ducts = pipeConvert(null, baseObj.elements!!.flexPipes,map)
            flexDucts = pipeConvert(baseObj.elements!!.ducts, null,map)
            connectors = connectorConvert(baseObj.elements!!.connectors,map)
        }


        // 处理Pipe
        val revitIdMap = HashMap<String, BaseComponent>() //     revit_id --> 对象
        result = result and RevitIdAnalyzer.analyzePipesDucts(
                dataService.pipeService as SObjectService<BaseComponent>,
                baseObj,
                //baseObj.elements!!.pipes as ArrayList<BaseComponent>?,
//                pipeConvert(null, baseObj.elements!!.pipes,map),
                pipes,
                revitIdMap,
                "Pipe"
        )
        if (!result) {
            return "处理Pipe异常"
        }
        // 处理flexPipe
        result = result and RevitIdAnalyzer.analyzePipesDucts(
                dataService.pipeService as SObjectService<BaseComponent>,
                baseObj,
//                baseObj.elements!!.flexPipes as ArrayList<BaseComponent>?,
//                pipeConvert(null, baseObj.elements!!.flexPipes,map),
                flexPipes,
                revitIdMap,
                "FlexPipe"
        )
        if (!result) {
            return "处理FlexPipe异常"
        }
        // 处理duct
        result = result and RevitIdAnalyzer.analyzePipesDucts(
                dataService.ductService as SObjectService<BaseComponent>,
                baseObj,
//                baseObj.elements!!.ducts as ArrayList<BaseComponent>?,
//                pipeConvert(baseObj.elements!!.ducts, null,map),
                ducts,
                revitIdMap,
                "Duct"
        )
        if (!result) {
            return "处理Duct异常"
        }
        // 处理flexDuct
        result = result and RevitIdAnalyzer.analyzePipesDucts(
                dataService.ductService as SObjectService<BaseComponent>,
                baseObj,
//                baseObj.elements!!.flexDucts as ArrayList<BaseComponent>?,
//                pipeConvert(baseObj.elements!!.flexDucts, null,map),
                flexDucts,
                revitIdMap,
                "FlexDuct"
        )
        if (!result) {
            return "处理FlexDuct异常"
        }

        // 处理other
        result = result and RevitIdAnalyzer.analyze(
                dataService.otherService as SObjectService<BaseComponent>,
                baseObj,
                baseObj.elements!!.others as ArrayList<BaseComponent>?,
                revitIdMap
        )
        if (!result) {
            return "处理other异常"
        }
        // 处理joinObject
        result = result and RevitIdAnalyzer.analyze(
                dataService.joinObjectService as SObjectService<BaseComponent>,
                baseObj,
                baseObj.elements!!.joinObjects as ArrayList<BaseComponent>?,
                revitIdMap
        )
        if (!result) {
            return "处理joinObject异常"
        }
        // 处理equipment
////        result = result and EquipmentAnalyzer.analyze(
////                dataService.equipmentService,
////                baseObj,
////                baseObj.elements!!.equipments,
////                bimIdPrefix, revitIdMap,
////                checkProject(baseObj.mbiName),
////                dataService.dictCompService,
////                dataService.infoPointCompService
////        )
        result = result and EquipmentAnalyzer.analyze(
                dataService.equipmentService,
                baseObj,
                baseObj.elements!!.equipments,
                bimIdPrefix, revitIdMap,
                checkProject,
                dataService.equipCompService,
                dataService.infoPointCompService,
                dataService.unrecognizedEquipService
        )

        if (!result) {
            return "处理equipment异常"
        }
        // 处理部件(component)
        result = result and EquipmentAnalyzer.analyzeComponent(
                dataService.componentService,
                baseObj,
                baseObj.elements!!.equipParts,
                bimIdPrefix,
                revitIdMap
        )
        if (!result) {
            return "处理component异常"
        }

        // 处理连接件
        result = result and RevitIdAnalyzer.analyzeConnector(
                dataService.connectorService,
                baseObj,
                //连接件转换
                connectors,
                bimIdPrefix,
                revitIdMap
        )
        if (!result) {
            return "处理连接件异常"
        }

        // 处理level, 标高
        result = result and BaseBehavier.analyze(
                dataService.levelService as SObjectService<BaseComponent>,
                baseObj,
                baseObj.elements!!.levels as ArrayList<BaseComponent>?
        )
        if (!result) {
            return "处理level异常"
        }
        // 处理grid, 轴网
        result = result and BaseBehavier.analyze(
                dataService.gridService as SObjectService<BaseComponent>,
                baseObj,
                baseObj.elements!!.grids as ArrayList<BaseComponent>?
        )
        if (!result) {
            return "处理grid异常"
        }
        return null
    }

    /**
     * 检查模型文件id是否存在
     */
    private fun checkModelId(modelId: String?): ModelFile? {
        modelId ?: return null
        val modelFileList = dataService.modelFileService.select(SFilter.eq("id", modelId)).exec()
        if (modelFileList.isEmpty())
            return null
        return modelFileList[0]
    }

    /**
     * 导入模型数据到数据库
     */
    fun importModelData(jsonData: BaseObj<AllComponents>, bimIdPrefix: String): SBaseResponse {
        try {
            val ret = handleSingleJSON(jsonData, bimIdPrefix)
            if (ret == null)
                return SBaseResponse(SResponseType.success)
            else
                return SBaseResponse(SResponseType.failure, ret)
        } catch (e: Exception) {
            return SBaseResponse(SResponseType.failure, "error:${e.message}, ${CommonService.getStackTrace(e)}")
        }
    }

    /**
     * 解析json数据
     */
    fun analyzeJsonzFile(fileStream: InputStream?): BaseObj<AllComponents>? {
        try {
            val map = UploadService.getJSONObjectMap(fileStream!!)
            val iterator = map.iterator()
            if (iterator.hasNext()) {
                return handleSingleJSON(iterator.next().value)
            } else {
                throw Exception("jsonz文件出错")
            }
        } catch (e: Exception) {
            throw Exception("jsonz文件出错: ${CommonService.getStackTrace(e)}")
        }
    }

    /**
     * 判断需要转换的项目
     */

    fun getProject(projectId: String?): Project? {
        if (projectId.isNullOrEmpty()) {
            return null
        }
        return dataService.projectService.select(SFilter.eq("id", projectId)).entity()
    }

    fun checkProject(projectId: String?): Boolean {
        if (projectId.isNullOrEmpty()) {
            return false
        }
        val projectConfig = getProject(projectId)
        return if(projectConfig == null){
            false
        }else projectConfig.type == "wanda"
    }

    /**
     * @Description: 管道类型转换
     */
    fun pipeConvert(ducts: ArrayList<Duct>?, pipes: ArrayList<Pipe>?,map: HashMap<String, String>): ArrayList<BaseComponent>? {
        if (ducts != null) {
            for (duct in ducts) {
                if(!duct.mepSystemType.isNullOrEmpty()){
                    if (map.containsKey(duct.mepSystemType!!)) {
                        duct.mepSystemType = map[duct.mepSystemType!!]
                    }
                }

            }
            return ducts as ArrayList<BaseComponent>?
        } else if (pipes != null) {
            for (pipe in pipes) {
                if(!pipe.mepSystemType.isNullOrEmpty()){
                    if (map.containsKey(pipe.mepSystemType!!)) {
                        pipe.mepSystemType = map[pipe.mepSystemType!!]
                    }
                }

            }
            return pipes as ArrayList<BaseComponent>?
        } else {
            return null
        }

    }

    /**
     * @Description: 连接件转换
     */
    fun connectorConvert(connectors: ArrayList<Connector>?,map: HashMap<String, String>): ArrayList<Connector>? {
        if (connectors != null) {
            for (connector in connectors) {
                if(!connector.mepSystemType.isNullOrEmpty()){
                    if (map.containsKey(connector.mepSystemType!!)) {
                        connector.mepSystemType = map[connector.mepSystemType!!]
                    }
                }

            }
        }
        return connectors
    }

}