/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.controller.task

import cn.hutool.core.bean.BeanUtil
import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.database.functions.SLength
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opts
import com.persagy.server.datacenter.models.entities.task.*
import com.persagy.server.datacenter.models.entities.task.base.ScanTaskBase
import com.persagy.server.datacenter.models.requests.ModelScanSQueryRequest
import com.persagy.server.services.task.EquipScanTaskService
import com.persagy.server.services.task.ModelScanTaskQueryService
import com.persagy.server.services.task.ModelScanTaskService
import com.persagy.server.services.task.ScanTaskBaseService
import com.persagy.service.SObjectService
import com.persagy.service.SPageContext
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.requests.SUpdateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.slf4j.LoggerFactory
import org.springframework.beans.BeanUtils
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController


/**
 * 模型任务接口
 *
 * @author  邓婷婷
 */
@Tag(name = "002、任务-模型任务接口")
@RestController
@RequestMapping("/task/model")
open class ModelScanTaskController {

    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(ModelScanTaskController::class.java)
    } // Companion object

    /**
     * 根据条件查询模型任务
     *
     * @author 邓婷婷
     */
    @Operation(summary = "查询模型任务", description = "级联查询对象：项目（project）、建筑（building）、楼层（floor）、方案（scanScheme）、设备分类（equipCategory）、部件（component）;" +
            "查询条件：任务类型（TaskSource--int）、任务执行情况（TaskState--int）、模型文件id（FileId）、设备类型（Category）")
    @PostMapping(value = ["/page-query"])
    fun pageQuery(@RequestBody request: SQueryRequest): SQueryResponse<ModelScanTaskQuery> {
        val equipScanTaskQueryService = SObjectService(SMybatisDao(ModelScanTaskQuery::class.java))
        val withFilterList = ArrayList<SFilter>()
        withFilterList.add(SFilter.eq("projectId", SPageContext.getHeader("projectId").toString()))
        withFilterList.add(SFilter.lt(SLength("classCode"), 7))
        return ModelScanTaskQueryService.pageQuerys(request);
    } // Function pageQuery()

    /**
     * 根据条件查询模型任务
     *
     * @author 邓婷婷
     */
    @Operation(summary = "查询模型任务，包含部件", description = "级联查询对象：项目（project）、建筑（building）、楼层（floor）、方案（scanScheme）、设备分类（equipCategory）、部件（component）;" +
            "查询条件：任务类型（TaskSource--int）、任务执行情况（TaskState--int）、模型文件id（FileId）、设备类型（Category）")
    @PostMapping(value = ["/page-query-com"])
    fun pageQueryCom(@RequestBody request: SQueryRequest): SQueryResponse<ModelScanTaskQuery> {
        logger.debug("modelScanTaskQuery =${request.toJson()}")
        return ModelScanTaskQueryService.pageQueryTwo(request)
    } // Function pageQuery()

    /**
     * 创建模型任务
     *
     * @param   request        模型任务对象列表
     * @return                      创建结果信息
     * @author  邓婷婷
     */
    @Operation(summary = "创建模型任务", description = "所有属性名要以查询返回结果为准  例如：任务id应用TaskId；当任务为未找到任务时重新生成任务，需要将TaskState=-1带入;")
    @PostMapping(value = ["/create"])
    fun create(@RequestBody request: SCreateRequest<ModelScanTask>): SCreateResponse<ModelScanTask> {
        logger.debug("equip =${request.toJson()}")
        val createBefore = ModelScanTaskService.createBefore(request)
        createModelScanTaskBase(createBefore)
        return ModelScanTaskService.createList(createBefore)
    } // Function create()


    /**
     * 同步创建基础模型台帐任务
     */
    fun createModelScanTaskBase(@RequestBody createBefore : SCreateRequest<ModelScanTask>){
        val scanTaskBase: ArrayList<ModelScanTask> = createBefore.content

        val sCreateRequest = SCreateRequest<ScanTaskBase>()
        var newList = arrayListOf<ScanTaskBase>()
        for(equip in scanTaskBase){
            val scanTaskBase =  ScanTaskBase()
            BeanUtils.copyProperties(equip,scanTaskBase)
            //val scanTaskBase: ScanTaskBase = equip
            scanTaskBase.projectId= Opts.projectId
            newList.add(scanTaskBase);
        }
        sCreateRequest.content = newList
        ScanTaskBaseService.createList(sCreateRequest)
    }
    /**
     * 更新模型任务
     *
     * @param   request      模型任务对象列表
     * @return              结果信息
     * @author  邓婷婷
     */
    @Operation(summary = "更新模型任务", description = "所有属性名要以查询返回结果为准  例如：任务id应用TaskId；")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<ModelScanTask>): SBaseResponse {
        updateScanTaskBase(request);
        return ModelScanTaskService.updateList(request)
    } // Function update()


    /**
     * 更新模型基础信息
     */
    fun updateScanTaskBase(@RequestBody request: SUpdateRequest<ModelScanTask>){
        //更新基础信息
        val sUpdateRequest = SUpdateRequest<ScanTaskBase>()
        var newList = arrayListOf<ScanTaskBase>()
        val equipList = request.content;
        if (equipList != null) {
            for(equip in equipList){
                val scanTaskBase: ScanTaskBase = equip
                newList.add(scanTaskBase);
            }
        }
        sUpdateRequest.content = newList
        ScanTaskBaseService.updateList(sUpdateRequest)
    }

    /**
     * 根据id删除模型任务
     *
     * @param   idList      id数组
     * @return              删除的结果信息
     *
     * @author  邓婷婷
     */
    @Operation(summary = "根据id删除模型任务", description = "所有属性名要以查询返回结果为准  例如：任务id应用TaskId；")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody idList: ArrayList<ModelScanTask>): SBaseResponse {
        var newList = arrayListOf<ScanTaskBase>()
        for(id in idList){
            newList.add(id);
        }
        ScanTaskBaseService.deleteByKeysList(newList)
        return ModelScanTaskService.deleteByKeysList(idList)
    } // Function delete()

    /**
     * 查询设备分类
     */
    @Operation(summary = "查询设备分类",description = "")
    @PostMapping(value = ["/model-category-query"])
    fun modelCategoryQuery(): SQueryResponse<EquipCategoryDic>{
        return ScanTaskBaseService.equipCategoryQuery("model")
    } // Function pageQuery()

    /**
     * 查询模型文件夹和文件
     */
    @Operation(summary = "查询模型文件夹和文件",description = "")
    @PostMapping(value = ["/model-file-query"])
    fun modelFileQuery(): SQueryResponse<ModelFolder>{
        return ModelScanTaskService.modelFileQuery()
    } // Function ModelFileQuery()

    /**
     * 查询建筑和楼层
     */
    @Operation(summary = "查询建筑和楼层",description = "")
    @PostMapping(value = ["/model-building-query"])
    fun buildingQuery(): SQueryResponse<BuildingDic>{
        return ScanTaskBaseService.buildingQuery("model")
    } // Function BuildingQuery()

    /**
     * 查询根据模型空间id查询设备信息
     */
    @Operation(summary = "查询根据模型空间id查询设备信息",description = "listId 模型id")
    @PostMapping(value = ["/equip-query"])
    fun equipQuery(@RequestBody request: ModelScanSQueryRequest): SQueryResponse<EquipmentTaskTwo>{
        return ModelScanTaskService.equipQuery(request)
    } // Function equipQuery()

    /**
     * 查询设备任务下的部件类型及数量
     */
    @Operation(summary = "查询设备任务下的部件类型及数量",description = " TaskId、EquipId 必填")
    @PostMapping(value = ["/category-count"])
    fun categoryCount(@RequestBody equipScanTask: ModelScanTask): SQueryResponse<CategoryStatistics>{
        return ModelScanTaskService.categoryCounts(equipScanTask)
    } // Fun

} // Class ModelScanTaskController