/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.controller.task

import cn.hutool.core.bean.BeanUtil
import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.server.Opts
import com.persagy.server.datacenter.models.entities.task.BuildingDic
import com.persagy.server.datacenter.models.entities.task.EquipCategoryDic
import com.persagy.server.datacenter.models.entities.task.ModelScanTask
import com.persagy.server.datacenter.models.entities.task.PropertyScanTask
import com.persagy.server.datacenter.models.entities.task.base.ScanTaskBase
import com.persagy.server.services.task.ModelScanTaskService
import com.persagy.server.services.task.PropertyScanTaskService
import com.persagy.server.services.task.ScanTaskBaseService
import com.persagy.service.SPageContext
import com.persagy.service.models.requests.SCountRequest
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.requests.SUpdateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCountResponse
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.slf4j.LoggerFactory
import org.springframework.beans.BeanUtils
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController


/**
 * 资产台账任务接口
 *
 * @author  邓婷婷
 */
@Tag(name = "003、任务-资产台账任务接口")
@RestController
@RequestMapping("/task/property")
open class PropertyScanTaskController {

    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(PropertyScanTaskController::class.java)
    } // Companion object

    /**
     * 根据条件查询资产台账任务
     *
     * @author 邓婷婷
     */
    @Operation(summary = "查询资产台账任务", description = "级联查询对象：项目（project）、建筑（building）、楼层（floor）、方案（scanScheme）、设备族（equipFamily）;" +
            "查询条件：任务类型（TaskSource--int）、任务执行情况（TaskState--int）、所属建筑（BuildingId）、所属楼层（FloorId）、设备类型（Category）")
    @PostMapping(value = ["/page-query"])
    fun pageQuery(@RequestBody request: SQueryRequest): SQueryResponse<PropertyScanTask> {
        logger.debug("propertyScanTaskQuery =${request.toJson()}")
        val withFilterList = ArrayList<SFilter>()
       // withFilterList.add(SFilter.eq("projectId", SPageContext.getHeader("projectId").toString()))
        return PropertyScanTaskService.pageQuerys(request);
    } // Function pageQuery()

    /**
     * 创建资产台账任务
     *
     * @param   request        资产台账任务对象列表
     * @return                      创建结果信息
     * @author  邓婷婷
     */
    @Operation(summary = "创建资产台账任务", description = "所有属性名要以查询返回结果为准  例如：任务id应用TaskId；当任务为未找到任务时重新生成任务，需要将TaskState=-1带入;")
    @PostMapping(value = ["/create"])
    fun create(@RequestBody request: SCreateRequest<PropertyScanTask>): SCreateResponse<PropertyScanTask> {
        logger.debug("equip =${request.toJson()}")
        val createBefore = PropertyScanTaskService.createBefore(request)
        createPropertyScanTaskBase(createBefore);
        return PropertyScanTaskService.createList(createBefore)
    } // Function create()


    /**
     * 同步创建基础资产台帐任务
     */
    fun createPropertyScanTaskBase(@RequestBody createBefore: SCreateRequest<PropertyScanTask>){
        val scanTaskBase: ArrayList<PropertyScanTask> = createBefore.content;
        val sCreateRequest = SCreateRequest<ScanTaskBase>()
        var newList = arrayListOf<ScanTaskBase>()
        for(equip in scanTaskBase){
            val scanTaskBase  = ScanTaskBase()
             BeanUtils.copyProperties(equip,scanTaskBase)
            scanTaskBase.projectId= Opts.projectId
            newList.add(scanTaskBase);
        }
        sCreateRequest.content = newList
        ScanTaskBaseService.createList(sCreateRequest)
    }

    /**
     * 更新资产台账任务
     *
     * @param   request      资产台账任务对象列表
     * @return              结果信息
     * @author  邓婷婷
     */
    @Operation(summary = "更新资产台账任务", description = "所有属性名要以查询返回结果为准  例如：任务id应用TaskId；")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<PropertyScanTask>): SBaseResponse {
        updateScanTaskBase(request)
        return PropertyScanTaskService.updateList(request)
    } // Function update()

    /**
     * 更新资产基础信息
     */
    fun updateScanTaskBase(@RequestBody request: SUpdateRequest<PropertyScanTask>){
        //更新基础信息
        val sUpdateRequest = SUpdateRequest<ScanTaskBase>()
        var newList = arrayListOf<ScanTaskBase>()
        val equipList = request.content;
        if (equipList != null) {
            for(equip in equipList){
                val scanTaskBase: ScanTaskBase = equip
                newList.add(scanTaskBase);
            }
        }
        sUpdateRequest.content = newList
        ScanTaskBaseService.updateList(sUpdateRequest)
    }
    /**
     * 根据id删除资产台账任务
     *
     * @param   idList      id数组
     * @return              删除的结果信息
     *
     * @author  邓婷婷
     */
    @Operation(summary = "根据id删除资产台账任务", description = "所有属性名要以查询返回结果为准  例如：任务id应用TaskId；")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody idList: ArrayList<PropertyScanTask>): SBaseResponse {
        var newList = arrayListOf<ScanTaskBase>()
        for(id in idList){
            newList.add(id);
        }
        ScanTaskBaseService.deleteByKeysList(newList)
        return PropertyScanTaskService.deleteByKeysList(idList)
    } // Function delete()

    /**
     * 统计数量
     */
    @Operation(summary = "统计数量", description = "查询未完成撕码任务TaskState=1&&TaskType='3' ，未完成换码任务TaskState=1&&TaskType='2'")
    @PostMapping(value = ["/count"])
    fun count(@RequestBody request: SCountRequest): SCountResponse {
        val withFilterList = ArrayList<SFilter>()
        withFilterList.add(SFilter.eq("projectId", SPageContext.getHeader("projectId").toString()))
        return PropertyScanTaskService.count(request,withFilterList)
    } // Function count()

    /**
     * 查询设备族
     */
    @Operation(summary = "查询设备族",description = "")
    @PostMapping(value = ["/property-category-query"])
    fun propertyCategoryQuery(): SQueryResponse<EquipCategoryDic>{
        return ScanTaskBaseService.equipCategoryQuery("property");
    } // Function PropertyCategoryQuery()

    /**
     * 查询建筑和楼层
     */
    @Operation(summary = "查询建筑和楼层",description = "")
    @PostMapping(value = ["/property-building-query"])
    fun buildingQuery(): SQueryResponse<BuildingDic>{
        return ScanTaskBaseService.buildingQuery("property");
    } // Function BuildingQuery()

} // Class PropertyScanTaskController