/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.task

import cn.hutool.core.collection.CollUtil
import com.alibaba.fastjson.JSONObject
import com.persagy.database.SFilter
import com.persagy.database.functions.SLength
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.dictnew.DefClass
import com.persagy.server.datacenter.models.entities.objects.Equipment
import com.persagy.server.datacenter.models.entities.task.CategoryStatistics
import com.persagy.server.datacenter.models.entities.task.EquipScanTaskQuery
import com.persagy.server.services.AdmBaseService
import com.persagy.server.services.DmpBaseService
import com.persagy.server.util.AdmEntityTransferUtil
import com.persagy.service.SObjectService
import com.persagy.service.SPageContext
import com.persagy.service.models.SCascadeQuery
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.responses.SQueryResponse
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import java.util.stream.Collectors

/**
 * 设备台账任务
 *
 * @author  邓婷婷
 */
object EquipScanTaskQueryService : SObjectService<EquipScanTaskQuery>(SMybatisDao(EquipScanTaskQuery::class.java))  { // Class EquipScanTaskService

    /** 字典平台对象类型 */
  //  private val defClassService = SBaseService(SMybatisDao(DefClass::class.java))
    // 日志
    private val logger = LoggerFactory.getLogger(EquipScanTaskQueryService::class.java)

    @Autowired
    private val dmpBaseService: DmpBaseService? = DmpBaseService()

    @Autowired
    private val admBaseService: AdmBaseService? = AdmBaseService()


    fun pageQuerys(request: SQueryRequest): SQueryResponse<EquipScanTaskQuery> {
        //查询设备台账任务
        val projectId = SPageContext.getHeader("projectId").toString()
        val withFilterList = ArrayList<SFilter>()
        withFilterList.add(SFilter.eq("projectId", SPageContext.getHeader("projectId").toString()))
        withFilterList.add(SFilter.lt(SLength("classCode"), 7))
        val pageQuery = pageQuery(request, withFilterList)
        if (!pageQuery.content.isNullOrEmpty()&&pageQuery.content!!.size>0){

            //级联查询设备相关信息:建筑，楼层，空间等
            val equipScanTaskList = pageQuery.content
            var cascadeQueryAdmEquip = arrayListOf<Equipment>()
            if (equipScanTaskList != null) {

                val cascades = ArrayList<SCascadeQuery>()
                //级联查询建筑
                val buildCacade = SCascadeQuery()
                buildCacade.name = "building"
                cascades.add(buildCacade)
                //级联查询楼层
                val floorCacade = SCascadeQuery()
                floorCacade.name="floor"
                cascades.add(floorCacade)
                request.cascade= cascades

                cascadeQueryAdmEquip = cascadeQueryAdmEquip(equipScanTaskList, request) as ArrayList<Equipment>
            }
            val equipMap = cascadeQueryAdmEquip?.stream()?.collect(Collectors.groupingBy { item: Equipment -> item.id })
            for (content in pageQuery.content!!){
                //设置级联查询结果
                val caseNodes = equipMap.get(content.equipId)
                if(CollUtil.isNotEmpty(caseNodes)){
                    val equip = caseNodes?.get(0)
                    if (equip != null) {
                        content.building = equip.building
                        content.floor = equip.floor
                        /*val component = equip.component
                        val toAdmMultiEntity = AdmEntityTransferUtil.toAdmMultiEntity(component, null, EquipScanTaskQuery::class.java)
                        content.component = toAdmMultiEntity*/
                    }
                }

                //查询组件
                val exec = select(SFilter.eq("parentId", content.equipId!!),SFilter.eq("taskParentId", content.id!!)).exec()
                if (exec.size>0){
                    content.component = exec
                }
                //查询中台类型数据，2021.10.25日中心中台整合调整
                val param = JSONObject()
                val criteria = JSONObject()
                criteria["code"] = content.classCode!!
                param["criteria"] = criteria
                val dictTypes =dmpBaseService!!.queryDictType(param)
                val defClass = AdmEntityTransferUtil.toAdmMultiEntity(dictTypes, null, DefClass::class.java)
                /*   val defClass =
                       defClassService.select(SFilter.eq("projectId", projectId), SFilter.eq("code", content.classCode!!)).entity()*/
                if (defClass!=null && defClass.size>0) {
                    content.equipCategory = defClass.get(0)
                }
            }
        }
        return pageQuery
    } // Fun

    fun pageQueryTwo(request: SQueryRequest): SQueryResponse<EquipScanTaskQuery> {
        val projectId = SPageContext.getHeader("projectId").toString()
        val withFilterList = ArrayList<SFilter>()
        withFilterList.add(SFilter.eq("projectId", SPageContext.getHeader("projectId").toString()))
        val pageQuery = pageQuery(request, withFilterList)
        if (!pageQuery.content.isNullOrEmpty()&&pageQuery.content!!.size>0){
            //级联查询设备相关信息:建筑，楼层，空间等
            val equipScanTaskList = pageQuery.content
            var cascadeQueryAdmEquip = arrayListOf<Equipment>()
            if (equipScanTaskList != null) {
                cascadeQueryAdmEquip = cascadeQueryAdmEquip(equipScanTaskList, request) as ArrayList<Equipment>
            }
            val equipMap = cascadeQueryAdmEquip?.stream()?.collect(Collectors.groupingBy { item: Equipment -> item.id })

            for (content in pageQuery.content!!){
                //设置级联查询结果
                val caseNodes = equipMap.get(content.equipId)
                if(CollUtil.isNotEmpty(caseNodes)){
                    val equip = caseNodes?.get(0)
                    if (equip != null) {
                        content.building = equip.building
                        content.floor = equip.floor
                        /*val component = equip.component
                        val toAdmMultiEntity = AdmEntityTransferUtil.toAdmMultiEntity(component, null, EquipScanTaskQuery::class.java)
                        content.component = toAdmMultiEntity*/
                    }
                }
                val exec = select(SFilter.eq("parentId", content.equipId!!),SFilter.eq("taskParentId", content.id!!)).exec()
                if (exec.size>0){
                    content.component = exec
                }
                //查询中台类型数据，2021.10.25日中心中台整合调整
                val param = JSONObject()
                val criteria = JSONObject()
                criteria["code"] = content.classCode!!
                param["criteria"] = criteria
                val dictTypes =dmpBaseService!!.queryDictType(param)
                val defClass = AdmEntityTransferUtil.toAdmMultiEntity(dictTypes, null, DefClass::class.java)
                /*   val defClass =
                       defClassService.select(SFilter.eq("projectId", projectId), SFilter.eq("code", content.classCode!!)).entity()*/
                if (defClass!=null && defClass.size>0) {
                    content.equipCategory = defClass.get(0)
                }
            }
        }
        return pageQuery
    } // Fun


    /**
     * 级联查询设备相关信息:建筑，楼层，空间等
     */
    fun cascadeQueryAdmEquip(equipScanTaskList: List<EquipScanTaskQuery>,request: SQueryRequest): List<Equipment>? {
        //级联查询设备相关信息:建筑，楼层，空间等
        if(CollUtil.isEmpty(equipScanTaskList)){
            return ArrayList<Equipment>()
        }
        val equipIds = equipScanTaskList?.stream()?.map(EquipScanTaskQuery::equipId)?.collect(Collectors.toList<String>())
        var equipRequest = SQueryRequest()
        equipRequest.filters = "id in "+"(\'" + java.lang.String.join("\',\'", equipIds) + "\')"

        val requestCascade = request.cascade
        //查询级联关系
        val cascadeList = arrayListOf<SCascadeQuery>()
        if (requestCascade != null) {
            for(cascade in requestCascade){
                if("scanScheme".equals(cascade.name)){
                    continue
                }
                cascadeList.add(cascade)
            }
            equipRequest.cascade = cascadeList;
            return admBaseService?.queryEquip(equipRequest)
        }
        return ArrayList<Equipment>()
    }



    /**
     * 分类统计
     */
    fun categoryStatistics(projectId:String,id: String,equipId: String): ArrayList<CategoryStatistics>{
        val result = ArrayList<CategoryStatistics>()
        //查询所有的设备类别
        val dictTypeList = dmpBaseService?.queryDictType(JSONObject())
        val dictTypeMap = HashMap<String, String>()
        if (dictTypeList != null) {
            for(dictType in dictTypeList){
                dictType.code?.let { dictType.name?.let { it1 -> dictTypeMap.put(it, it1) } }
            }
        }

        //查询设备任务
        val request = SQueryRequest()
        val withFilterList = ArrayList<SFilter>()
        withFilterList.add(SFilter.eq("projectId", projectId))
        withFilterList.add(SFilter.eq("taskParentId", id))
        withFilterList.add(SFilter.eq("parentId", equipId))
        val pageQuery = EquipScanTaskQueryService.pageQuery(request, withFilterList)
        val equipScanTask = pageQuery.content

        if (!pageQuery.content.isNullOrEmpty()&&pageQuery.content!!.size>0) {
            val equipScanTaskMap: Map<String?, List<EquipScanTaskQuery>> = equipScanTask?.stream()
                    ?.collect(Collectors.groupingBy(EquipScanTaskQuery::classCode)) as Map<String?, List<EquipScanTaskQuery>>

            for ((key, value) in equipScanTaskMap.entries) {
                val categoryStatistics = CategoryStatistics()
                categoryStatistics.code = key
                categoryStatistics.name = dictTypeMap.get(key)
                categoryStatistics.count = value.size
                result.add(categoryStatistics)
            }

        }
        return result
    }



}