package cn.sagacloud.server.scheduler.controllers

import cn.sagacloud.server.DispatchTask
import cn.sagacloud.server.scheduler.models.*
import io.swagger.annotations.Api
import io.swagger.annotations.ApiOperation
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController
import cn.sagacloud.server.scheduler.pojo.Command
import cn.sagacloud.server.scheduler.service.BaseDataService
import cn.sagacloud.server.scheduler.service.TasksService
import cn.sagacloud.server.scheduler.utils.CommonUtil
import com.alibaba.fastjson.JSON
import com.sybotan.base.extensions.toJson
import com.sybotan.database.SFilter
import com.sybotan.mybatis.SMybatisDao
import com.sybotan.service.SObjectService
import com.sybotan.service.models.enums.SResponseType
import com.sybotan.service.models.requests.SCreateRequest
import com.sybotan.service.models.requests.SQueryRequest
import com.sybotan.service.models.requests.SUpdateRequest
import com.sybotan.service.models.responses.*
import org.springframework.beans.factory.annotation.Autowired


@Api(tags = ["Task接口"])
@RestController
@RequestMapping("/task")
open class TaskController{
    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(TaskController::class.java)
    } // Companion object

    @Autowired
    lateinit var tasksService : TasksService

    /**
     * 被动派发Task任务
     *
     * @param   request     请求
     * @return  派发结果应答信息
     */
    @ApiOperation(value="派发Task任务", notes="输入为下载模型文件的md5集合")
    @PostMapping(value = ["/dispatch"])
    fun dispatch(@RequestBody request: ArrayList<String>): SBaseResponse {
        val allTaskMap = tasksService.getAllTaskMapByStatus(arrayListOf(0, 1))
        if(allTaskMap.isEmpty()){
            return SBaseResponse()
        }
        // 匹配到MD5的任务
        val taskEntity = tasksService.getTaskByMD5(request, allTaskMap)
        val dispatchTaskResponse = DispatchTaskResponse()
        dispatchTaskResponse.result = SResponseType.success
        if(taskEntity == null){
//            dispatchTaskResponse.tasks.add(allTaskMap[0]!!)
            dispatchTaskResponse.tasks.add(allTaskMap.values.iterator().next())
        }else
            dispatchTaskResponse.tasks.add(taskEntity)
        return dispatchTaskResponse
    } // Function update()

    /**
     * 客户端确认接受Task任务
     *
     * @param   request     请求
     * @return  结果应答信息
     */
    @ApiOperation(value="确认Task任务", notes="输入为任务Id, LastClient, 暂时只支持单个任务的确认")
    @PostMapping(value = ["/confirm"])
    fun confirmDispatch(@RequestBody request: TaskEntity): SBaseResponse {
        val response = SBaseResponse()
        request.status = 2
        val updateResult = BaseDataService.taskService.update(request, arrayListOf("status", "lastClient"), SFilter.inList("status", (arrayListOf(0, 1))))
        if(updateResult){
            response.result = SResponseType.success
        }
        return response
    } // Function update()

    /**
     * 客户端提交Task任务结果
     *
     * @param   request     请求
     * @return  结果应答信息
     */
    @ApiOperation(value="提交Task任务结果", notes="输入为任务Id, TaskResultJson, ClientInfo, 暂时只支持单个任务的提交")
    @PostMapping(value = ["/commit"])
    fun commit(@RequestBody request: CommitTaskRequest): SBaseResponse {
        val response = SBaseResponse()
        if(request.command == null)
            return response
        when(request.command){
            Command.CommandError -> {
                request.status = 4
            }
            Command.DownloadError -> {
                request.status = 3
            }
            Command.TaskSuccess -> {
                request.status = 5
            }
        }
        DispatchTask.processResult(request, request.resultJson?:"")
        val updateResult = BaseDataService.taskService.update(request, arrayListOf("status", "resultJson", "clientInfo"), SFilter.Companion.inList("status", arrayListOf(0, 1, 2, 3, 4)))
        if(updateResult)
            response.result = SResponseType.success
        return response
    } // Function update()


    /**
     * 创建Task
     */
    @ApiOperation(value = "创建Task", notes = "")
    @PostMapping(value = ["/create"])
    fun create(@RequestBody taskEntity: SCreateRequest<TaskEntity>): SCreateResponse<TaskEntity> {
        //logger.debug("download =${taskEntity.toJson()}")
//        val response = taskService.createList(taskEntity)
        taskEntity.content.forEach {
            if(it.id == null){
                it.id = CommonUtil.generateUUID("", true)
            }
        }
        return BaseDataService.taskService.createList(taskEntity)
    } // Function create()

    /**
     * 更新Task
     *
     * @param   request     更新请求
     * @return  更新结果应答信息
     */
    @ApiOperation(value="更新Task", notes="")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<TaskEntity>): SBaseResponse {
        return BaseDataService.taskService.updateList(request)
    } // Function update()

    /**
     * 查询Task
     *
     * @param   request     查询请求
     * @return  查询结果应答信息
     */
    @ApiOperation(value="查询Task", notes="")
    @PostMapping(value = ["/query"])
    fun query(@RequestBody request: SQueryRequest): SQueryResponse<TaskEntity> {
        return BaseDataService.taskService.pageQuery(request)
    } // Function query()


    /**
     * 删除Task
     *
     * @param   idList      TaskEntity ID列表
     * @return  删除结果应答信息
     */
    @ApiOperation(value = "删除Task", notes = "")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody idList: ArrayList<TaskEntity>): SBaseResponse {
        return BaseDataService.taskService.deleteByKeysList(idList)
    } // Function delete()
}